import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  select,
  scaleBand,
  scaleLinear,
  max,
  axisBottom,
  axisLeft,
} from 'd3';

const csvUrl =
  'https://gist.githubusercontent.com/SirishaManthena/00d75ea671e22464684959a3641fe4dd/raw/43693d04538f604966ed86bbbdf1a8fcd9e7f251/CarSales.csv';
const width = 800;
const height = 500;
const margin = {
  top: 20,
  right: 20,
  bottom: 50,
  left: 100,
};

const BarChart = ({ data }) => {
  const xValue = (d) => d['Model'].split(',')[0];
  const yValue = (d) => +d['Price ($)'];

  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const xScale = scaleBand()
    .domain(data.map(xValue))
    .range([0, innerWidth])
    .padding(0.1);

  const yScale = scaleLinear()
    .domain([0, max(data, yValue)])
    .range([innerHeight, 0]);

  useEffect(() => {
    const xAxis = axisBottom(xScale);
    const yAxis = axisLeft(yScale).tickFormat(
      (d) => `$${d}`,
    );

    select('.x-axis').call(xAxis);
    select('.y-axis').call(yAxis);
  }, [xScale, yScale]);

  return (
    <svg width={width} height={height}>
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        {data.map((d) => (
          <rect
            key={xValue(d)}
            x={xScale(xValue(d))}
            y={yScale(yValue(d))}
            height={innerHeight - yScale(yValue(d))}
            width={xScale.bandwidth()}
            fill="#69b3a2"
          />
        ))}
        <g
          transform={`translate(0,${innerHeight})`}
          className="x-axis"
        />
        <g className="y-axis" />
      </g>
    </svg>
  );
};

const App = () => {
  const [data, setData] = useState(null);

  useEffect(() => {
    const row = (d) => {
      d['Price ($)'] = +d['Price ($)'];
      return d;
    };
    csv(csvUrl, row).then(setData);
  }, []);

  return (
    <div>
      <h1>Top Sold Car Brands</h1>
      {data && <BarChart data={data} />}
    </div>
  );
};

ReactDOM.render(<App />, document.getElementById('root'));
